package com.mozgoweb.demo.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.gadgets.client.Gadget;
import com.google.gwt.gadgets.client.Gadget.AllowHtmlQuirksMode;
import com.google.gwt.gadgets.client.Gadget.InjectModulePrefs;
import com.google.gwt.gadgets.client.Gadget.ModulePrefs;
import com.google.gwt.gadgets.client.Gadget.UseLongManifestName;
import com.google.gwt.gadgets.client.GadgetFeature;
import com.google.gwt.gadgets.client.UserPreferences;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.ui.HTML;

/**
 * Main entry point.
 *
 * @author Denis Lunev <den@mozgoweb.com>
 *
 * Twitter functionality based on "Cross-Site Scripting with GWT and Twitter"
 * http://jectbd.com/?p=406 by Alex Moffat
 *
 */
@ModulePrefs(title = "DemoGadget", author = "Denis Lunev", author_email = "den@mozogweb.com")
@UseLongManifestName(false)
@AllowHtmlQuirksMode(false)
@GadgetFeature.FeatureName(value = "google.contentmatch")
@InjectModulePrefs(files = {"Extractors.xml"})
public class CtxGadgetEntryPoint extends Gadget<UserPreferences> {
    public CtxGadgetEntryPoint() {}

    /*
     For gadget use init() instead of onModuleLoad 
     */
    @Override
    protected void init(UserPreferences preferences) {
        JSONObject[] objs = getContentMatches();
        String body = "";
        for (JSONObject ob : objs) {
            
            if(ob.get("email_body") != null){
                body = ob.get("email_body").toString();
            }
        }

        demoGadget(body);
    }

    //Native JS method to invoke extractors
    native JsArray<JavaScriptObject> jsGetContentMatches() /*-{
        return google.contentmatch.getContentMatches();
    }-*/;

    private JSONObject[] getContentMatches() {
        JSONObject[] result = null;
        final JsArray<JavaScriptObject> matches = jsGetContentMatches();
        if (matches != null) {
            final int numMatches = matches.length();
            result = new JSONObject[numMatches];
            for (int i = 0; i < numMatches; i++) {
                final JavaScriptObject jso = matches.get(i);

                result[i] = new JSONObject(jso);
            }
        }
        return result;
    }


    private void demoGadget(String message){

        String[] m = getUsers(message);
        for (int i = 0; i < m.length; i++) {
            String usr = m[i].replace("@", "");
            getJson(i, usr, this);
        }
    }

   private static native void getJson(int requestId, String twitterUser, CtxGadgetEntryPoint handler) /*-{

    // [0] The name of the callback function that will get called.
    var callback = "callback" + requestId;

    // [1] Create a script element.
    var script = document.createElement("script");
    script.setAttribute("src",
            "http://api.twitter.com/1/users/show/" + twitterUser +
                ".json?callback=" + callback);
    script.setAttribute("type", "text/javascript");

    // [2] Add a callback function on the window object.
    window[callback] = function(jsonObj) {
        // [3] Call the handleJsonResponseMethod.
        handler.@com.mozgoweb.demo.client.CtxGadgetEntryPoint::handleJsonResponse(ILcom/google/gwt/core/client/JavaScriptObject;)(requestId, jsonObj);
        window[callback + "done"] = true;
    };

    // [4] Wait up to 5 seconds for the call to return.
    setTimeout(function() {
        if (!window[callback + "done"]) {
            handler.@com.mozgoweb.demo.client.CtxGadgetEntryPoint::handleJsonResponse(ILcom/google/gwt/core/client/JavaScriptObject;)(requestId, null);
        }

        // [5] Cleanup. Remove script and callback elements.
        document.body.removeChild(script);
        delete window[callback];
        delete window[callback + "done"];
    }, 5000);

    // [6] Attach the script element to the document body.
    document.body.appendChild(script);
}-*/;

    private static native String[] getUsers(String txt)/*-{
        var re = new RegExp("@(\\w+)", "g");
        var m = txt.match(re);
        return m;
    }-*/;


    private void handleJsonResponse(int requestId, JavaScriptObject user){

        if(user != null){
            JSONObject js = new JSONObject(user);
            StringBuilder sb = new StringBuilder();
            sb.append("<table><tr><td>");
            sb.append("<img src=\"").append(clean(js.get("profile_image_url").toString())).append("\" />");
            sb.append("</td><td>");
            sb.append("<b>").append(clean(js.get("screen_name").toString())).append("</b><br />");
            sb.append(clean(js.get("description").toString()));
            sb.append("</td></tr></table>");

            RootPanel.get().add(new HTML(sb.toString()));
        }
    }

    private String clean(String txt){
        return txt.replaceAll("\"", "").trim();
    }

}
